//
//  ViewController.swift
//  Angry Birds en Tierra y Luna
//
//  Created by Santiago Pavón on 28/9/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import UIKit

class ViewController: UIViewController,  TrajectoryViewDataSource {

    @IBOutlet weak var speedSlider: UISlider!
    @IBOutlet weak var angleSlider: UISlider!
    @IBOutlet weak var distanceSlider: UISlider!
    @IBOutlet weak var scaleSlider: UISlider!
    
    @IBOutlet weak var speedValue: UILabel!
    @IBOutlet weak var angleValue: UILabel!
    @IBOutlet weak var distanceValue: UILabel!
    @IBOutlet weak var scaleValue: UILabel!

    
    @IBOutlet weak var earthTrajView: TrajectoryView!
    @IBOutlet weak var moonTrajView: TrajectoryView!
    
    var earthTrajModel: TrajectoryModel!
    var moonTrajModel: TrajectoryModel!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        earthTrajModel = TrajectoryModel(name: "La Tierra", gravity: 9.80665)
        moonTrajModel = TrajectoryModel(name: "La Luna", gravity: 1.622)
        
        earthTrajView.dataSource = self
        moonTrajView.dataSource = self
        
        speedSlider.sendActionsForControlEvents(.ValueChanged)
        angleSlider.sendActionsForControlEvents(.ValueChanged)
        distanceSlider.sendActionsForControlEvents(.ValueChanged)
        scaleSlider.sendActionsForControlEvents(.ValueChanged)
    
    }

    // MARK: IB Actions

    @IBAction func speedChanged(sender: UISlider) {
        earthTrajModel.speed = Double(sender.value)
        moonTrajModel.speed = Double(sender.value)
        
        speedValue.text = String(format: "%.2f m/s", arguments: [sender.value])
        
        earthTrajView.setNeedsDisplay()
        moonTrajView.setNeedsDisplay()
    }

    @IBAction func angleChanged(sender: UISlider) {
        earthTrajModel.angle = Double(sender.value)
        moonTrajModel.angle = Double(sender.value)
        
        angleValue.text = String(format: "%.2f rad", arguments: [sender.value])
        
        earthTrajView.setNeedsDisplay()
        moonTrajView.setNeedsDisplay()
    }
    
    @IBAction func distanceChanged(sender: UISlider) {
        distanceValue.text = String(format: "%.2f m", arguments: [sender.value])
        
        earthTrajView.distanceToTarget = Double(sender.value)
        moonTrajView.distanceToTarget = Double(sender.value)
    }

    @IBAction func scaleChanged(sender: UISlider) {
        let scale = Double(sender.value)
        scaleValue.text = String(format: "%.2f mpp", arguments: [scale])
        earthTrajView.scale = scale
        moonTrajView.scale = scale
    }

    // MARK: TrajectoryViewDataSource {

    func startTimeOfTrajectoryView(trajectoryView: TrajectoryView) -> Double {
        return 0
    }
    
    func endTimeOfTrajectoryView(trajectoryView: TrajectoryView) -> Double {
        
        switch trajectoryView {
        case earthTrajView:
            return earthTrajModel.timeToTarget()
        case moonTrajView:
            return moonTrajModel.timeToTarget()
        default:
            return 0.0
        }
    }
    
    func positionOfTrajectoryView(trajectoryView: TrajectoryView, atTime time: Double) -> Position {
        
        switch trajectoryView {
        case earthTrajView:
            let distance = earthTrajModel.distanceAtTime(time)
            let alture = earthTrajModel.altureAtTime(time)
            return Position(distance: distance, alture: alture)
        case moonTrajView:
            let distance = moonTrajModel.distanceAtTime(time)
            let alture = moonTrajModel.altureAtTime(time)
            return Position(distance: distance, alture: alture)
        default:
            return Position(distance: 0, alture: 0)
        }
    }
}

